//========================================================================
//    NeHe OpenGL Wizard : Example.cpp
//    Wizard Created by: Vic Hollis
//========================================================================
/**************************************
*                                     *
*      Jeff Molofee's Basecode        *
*          nehe.gamedev.net           *
*                2001                 *
*                                     *
**************************************/

#include <windows.h>											// Header File For Windows											// Header File For The OpenGL32 Library											// Header File For The GLu32 Library
#include <gl/glew.h>
#include <gl/wglew.h>											// Header File For The GLaux Library
#include <time.h> 
#include <stdio.h>
#include "global.h"	
#include "resource.h"
#pragma comment( lib, "glew32.lib" )								// Search For GLaux.lib While Linking
#pragma comment( lib, "glew32s.lib" )	
#ifndef CDS_FULLSCREEN											// CDS_FULLSCREEN Is Not Defined By Some
#define CDS_FULLSCREEN 4										// Compilers. By Defining It This Way,
#endif															// We Can Avoid Errors

GL_Window*	g_window;
Keys*		g_keys;
// User Defined Variables		
#define NP 500		// Number of Particles
int loop;			// Multi-purpose variable
float rotm,rv,re,rm,rj,rs,rn,ru,rp,r;	//Various variables
//Light Variables
GLfloat light[]={1.0f,1.0f,1.0f,1.0f};	
GLfloat LP[]={0.0,0.0,-100.0,1.0f};

p particle[NP];		//Particle Variable
GLuint texture[18];	//Texture Holder
GLUquadricObj *sphere;			//Sphere Holder


void LoadGLTextures()											// Creates Textures From Bitmaps In The Resource File
{
	HBITMAP hBMP;												// Handle Of The Bitmap
	BITMAP	BMP;												// Bitmap Structure

	// The ID Of The 3 Bitmap Images We Want To Load From The Resource File
	byte	Texture[]={IDB_GLOW,IDB_GLOWM,IDB_MERCURY,IDB_VENUS,IDB_EARTH,IDB_MARS,
						IDB_JUPITER,IDB_SATURN,IDB_URANUS,IDB_NEPTUNE,IDB_PLUTO,IDB_MOON,
						IDB_IO,IDB_GANYMEDE,IDB_CALLISTO,IDB_EUROPA,IDB_OBERON,IDB_RINGS};
					
	glGenTextures(sizeof(Texture), &texture[0]);				// Generate 3 Textures (sizeof(Texture)=3 ID's)
	for (int loop=0; loop<sizeof(Texture); loop++)				// Loop Through All The ID's (Bitmap Images)
	{
		hBMP=(HBITMAP)LoadImage(GetModuleHandle(NULL),MAKEINTRESOURCE(Texture[loop]), IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION);
		if (hBMP)												// Does The Bitmap Exist?
		{														// If So...
			GetObject(hBMP,sizeof(BMP), &BMP);					// Get The Object
																// hBMP: Handle To Graphics Object
																// sizeof(BMP): Size Of Buffer For Object Information
																// Buffer For Object Information
			glPixelStorei(GL_UNPACK_ALIGNMENT,4);				// Pixel Storage Mode (Word Alignment / 4 Bytes)
			glBindTexture(GL_TEXTURE_2D, texture[loop]);		// Bind Our Texture
			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);	// Linear Filtering
			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR); // Mipmap Linear Filtering

			// Generate Mipmapped Texture (3 Bytes, Width, Height And Data From The BMP)
			gluBuild2DMipmaps(GL_TEXTURE_2D, 3, BMP.bmWidth, BMP.bmHeight, GL_BGR_EXT, GL_UNSIGNED_BYTE, BMP.bmBits);
			DeleteObject(hBMP);									// Delete The Bitmap Object
		}
	}
}
BOOL Initialize (GL_Window* window, Keys* keys)					// Any GL Init Code & User Initialiazation Goes Here
{
	g_window	= window;
	g_keys		= keys;
	
	LoadGLTextures();											// Load the Textures
	
	glClearColor (0.0f, 0.0f, 0.0f, 0.5f);						// Black Background
	glClearDepth (1.0f);										// Depth Buffer Setup
	glDepthFunc (GL_LEQUAL);									// The Type Of Depth Testing (Less Or Equal)
	glEnable (GL_DEPTH_TEST);									// Enable Depth Testing
	glShadeModel (GL_SMOOTH);									// Select Smooth Shading
	glHint (GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);			// Set Perspective Calculations To Most Accurate

	glEnable(GL_TEXTURE_2D);									// Enable 2D Textures

	// Lighting Setup
	glLightfv(GL_LIGHT0,GL_AMBIENT,light);						
	glLightfv(GL_LIGHT0,GL_DIFFUSE,light);
	glLightfv(GL_LIGHT0,GL_POSITION,LP);
	glEnable(GL_LIGHT0);										// Enable Light0
	glEnable(GL_LIGHTING);										// Enable Lighting

	//Quadric Sphere Setup
	sphere=gluNewQuadric();										// Initialize our quadric
    gluQuadricTexture(sphere, GLU_TRUE);						// Textured
    gluQuadricDrawStyle(sphere, GLU_FILL);						// Filled

	//Particle Setup
	srand(time(NULL));											// New Random Seed
	for (loop=0;loop<NP;loop++)									// Initialize our particles
	{
		particle[loop].active=true;							
		particle[loop].x=float((rand()%-180)-90);	
		particle[loop].y=float((rand()%-150)-75);		
	}

	glEnable(GL_CULL_FACE);										// Enable Face Culling
	glCullFace(GL_BACK);										// Cut Out Back Faces
	return TRUE;												// Return TRUE (Initialization Successful)
}
void Deinitialize (void)										// Any User DeInitialization Goes Here
{
gluDeleteQuadric(sphere);
}

void Update (DWORD milliseconds)								// Perform Motion Updates Here
{
	if (g_keys->keyDown [VK_ESCAPE] == TRUE)					// Is ESC Being Pressed?
	{
		TerminateApplication (g_window);						// Terminate The Program
	}

	if (g_keys->keyDown [VK_F1] == TRUE)						// Is F1 Being Pressed?
	{
		ToggleFullscreen (g_window);							// Toggle Fullscreen Mode
	}
	//Make the planets rotate around the sun
	rotm+=(float) milliseconds/29.0f;
	rv+=(float)milliseconds/38.0f;
	re+=(float)milliseconds/43.0f;
	rm+=(float)milliseconds/45.0f;
	rj+=(float)milliseconds/52.0f;
	rs+=(float)milliseconds/54.0f;
	ru+=(float)milliseconds/56.0f;
	rn+=(float)milliseconds/57.0f;
	rp+=(float)milliseconds/58.0f;
	r+=(float)milliseconds/80.0f;

}

void Draw (void)
{
glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);			//Clear the Color and Depth Buffers
glDisable(GL_LIGHTING);										//Disable Lighting for the Stars
glColor3f(0.5f,0.5f,0.5f);									//Color the Stars Gray
glLoadIdentity();											
glTranslatef(0,0,-150);
for (loop=0;loop<NP;loop++)					// Loop Through All The Particles
	{
			float x=particle[loop].x;						// Grab Our Particle X Position
			float y=particle[loop].y;						// Grab Our Particle Y Position
			glDisable(GL_TEXTURE_2D);						// Disable Textures for the Stars
			glBegin(GL_POINTS);								// Draw the Points for the Stars
			glVertex3f(x,y,-15); 
			glEnd();										// Done Drawing Points
			glEnable(GL_TEXTURE_2D);						// Enable Texturing 
	}
glEnable(GL_LIGHTING);										// Enable Lighting
glColor3f(1,1,1);											// Color is White
glLoadIdentity();											
gluLookAt(0,30,110,0,0,0,0,1,0);
glBindTexture(GL_TEXTURE_2D, texture[1]);					// Which Texture
//////////////////////////////////////////////
/////////////////MERCURY//////////////////////
//////////////////////////////////////////////
glPushMatrix();												// Save the Matrix
glBindTexture(GL_TEXTURE_2D, texture[2]);					// Which Texture
glColor3f(1,1,1);											// White Color
glRotatef(rotm,0,1,0);										// Rotation Around the Sun
glTranslatef(0,0,5.79f);									// Translation
glRotatef(105,-1,0,0);										// Rotation to Make Planet Tilt The Right Way
gluSphere(sphere,0.267f,30,30);								// Draw Our Planet
glPopMatrix();												// Load the Former Matrix
//////////////////////////////////////////////
/////////////////VENUS////////////////////////
//////////////////////////////////////////////
glPushMatrix();												// Save the Matrix
glBindTexture(GL_TEXTURE_2D,texture[3]);					// Which Texture
glRotatef(rv,0,1,0);										// Rotation around the Sun
glTranslatef(0,0,-10.8f);									// Translation
glRotatef(75,-1,0,0);										// Tilt of the Planet
gluSphere(sphere,0.664f,30,30);								// Draw the Planet
glPopMatrix();												// Load the Former Matrix
//////////////////////////////////////////////
/////////////////EARTH////////////////////////
//////////////////////////////////////////////
glPushMatrix();												// Save the Matrix
glBindTexture(GL_TEXTURE_2D,texture[4]);					// Which Texture
glRotatef(re,0,1,0);										// Rotation around the Sun
glTranslatef(14.96f,0,0);									// Translation
glRotatef(105,-1,0,0);										// Tilt of the Planet
gluSphere(sphere,0.699f,30,30);								// Draw the Planet
/////////////////MOON//////////////////////////
glBindTexture(GL_TEXTURE_2D,texture[11]);					// New Texture for Moon
glTranslatef(0,0,0);										// Translation
glRotatef(re,0,0,1);										// Rotation Around Earth
glTranslatef(0,1.5,0.0f);									// Translation
gluSphere(sphere,0.2,30,30);								// Draw the Moon
glPopMatrix();												// Load the Former Matrix
//////////////////////////////////////////////
//////////////////MARS////////////////////////
//////////////////////////////////////////////				
glPushMatrix();												// Save the Matrix
glBindTexture(GL_TEXTURE_2D,texture[5]);					// Which Texture
glRotatef(rm,0,1,0);										// Rotation around the Sun
glTranslatef(-22.7f,0,0);									// Translation
glRotatef(105,1,0,0);										// Tilt of the planet
gluSphere(sphere,0.471f,30,30);								// Draw the Planet
glPopMatrix();												// Load the Former Matrix
//////////////////////////////////////////////
//////////////////JUPITER/////////////////////
//////////////////////////////////////////////
glPushMatrix();												// Save the Matrix											
glBindTexture(GL_TEXTURE_2D,texture[6]);					// Which Texture
glRotatef(rj,0,1,0);										// Rotation around the Sun
glTranslatef(30.94f,0,0);									// Translation
glRotatef(105,-1,0,0);										// Tilt of the Planet
gluSphere(sphere,1.914,30,30);								// Draw the Planet
//////////////////MOONS///////////////////////
glTranslatef(0,0,0);										// Translation
glRotatef(rj,0,0,1);										// Rotation around the Planet
glPushMatrix();												// Save the Matrix
glBindTexture(GL_TEXTURE_2D,texture[12]);					// Which Texture
glTranslatef(0,2.5,0);										// Translation
gluSphere(sphere,0.2,30,30);								// Draw the Moon
glPopMatrix();												// Load the Former Matrix
glPushMatrix();												// Save the Matrix
glBindTexture(GL_TEXTURE_2D,texture[13]);					// Which Texture
glTranslatef(2.5,0,0);										// Translation
gluSphere(sphere,0.3,30,30);								// Draw the Moon
glPopMatrix();												// Load the Former Matrix
glPushMatrix();												// Save the Matrix
glBindTexture(GL_TEXTURE_2D,texture[14]);					// Which Texture
glTranslatef(2.5,2.5,0);									// Translation
gluSphere(sphere,0.3,30,30);								// Draw the Moon
glPopMatrix();												// Load the Former Matrix
glPushMatrix();												// Save the Matrix
glBindTexture(GL_TEXTURE_2D,texture[15]);					// Which Texture
glTranslatef(0,-2.5,0);										// Translation
gluSphere(sphere,0.5,30,30);								// Draw the Moon
glPopMatrix();												// Load the Former Matrix
glPopMatrix();												// Load the Former Matrix
//////////////////////////////////////////////
//////////////////URANUS//////////////////////
//////////////////////////////////////////////
glPushMatrix();												// Save the Matrix
glBindTexture(GL_TEXTURE_2D,texture[8]);					// Which Texture
glRotatef(ru,0,1,0);										// Rotation around the Sun
glTranslatef(0,0,48.94f);									// Translation
glRotatef(105,-1,0,0);										// Tilt of the Planet
gluSphere(sphere,1.288,30,30);								// Draw the Planet
//////////////////MOONS///////////////////////
glTranslatef(0,0,0);										// Translation
glRotatef(ru,0,0,1);										// Rotation around the Planet
glPushMatrix();												// Save the Matrix
glBindTexture(GL_TEXTURE_2D,texture[16]);					// Which Texture
glTranslatef(2.0,0.0,0.0);									// Translation
gluSphere(sphere,0.3,30,30);								// Draw the Moon
glPopMatrix();												// Load the Former Matrix
glPushMatrix();												// Save the Matrix
glBindTexture(GL_TEXTURE_2D,texture[16]);					// Which Texture
glTranslatef(2.0,2.0,0.0);									// Translation
gluSphere(sphere,0.123,30,30);								// Draw the Moon
glPopMatrix();												// Load the Former Matrix
glPushMatrix();												// Save the Matrix
glBindTexture(GL_TEXTURE_2D,texture[16]);					// Which Texture
glTranslatef(0.0,2.0,0.0);									// Translation
gluSphere(sphere,0.4,30,30);								// Draw the Moon
glPopMatrix();												// Load the Former Matrix
glPushMatrix();												// Save the Matrix
glBindTexture(GL_TEXTURE_2D,texture[16]);					// Which Texture
glTranslatef(-2.0,-2.0,0.0);								// Translation
gluSphere(sphere,0.2,30,30);								// Draw the Moon
glPopMatrix();												// Load the Former Matrix
glPopMatrix();												// Load the Former Matrix
//////////////////////////////////////////////
/////////////////NEPTUNE//////////////////////
//////////////////////////////////////////////
glPushMatrix();												// Save the Matrix
glBindTexture(GL_TEXTURE_2D,texture[9]);					// Which Texture
glRotatef(rn,0,1,0);										// Rotation around the Sun
glTranslatef(0,0,-55.94f);									// Translation
glRotatef(75,-1,0,0);										// Tilt of the Planet
gluSphere(sphere,1.1,30,30);								// Draw the Planet
glPopMatrix();												// Load the Former Matrix
//////////////////////////////////////////////
///////////////////PLUTO//////////////////////
//////////////////////////////////////////////
glPushMatrix();												// Save the Matrix
glBindTexture(GL_TEXTURE_2D,texture[10]);					// Which Texture
glRotatef(rp,0,1,0);										// Rotation around the Sun
glTranslatef(0,0,-59.94f);									// Translation
glRotatef(75,-1,0,0);										// Tilt of the Planet
gluSphere(sphere,0.224,30,30);								// Draw the Planet
//////////////////MOONS///////////////////////
glTranslatef(0,0.4f,0);										// Translation
glRotatef(rp,0,0,1);glRotatef(rp,0,0,1);					// Rotation around the Planet
gluSphere(sphere,0.1,30,30);								// Draw the Moon
glPopMatrix();												// Load the Former Matrix
//////////////////////////////////////////////
//////////////////SATURN//////////////////////
//////////////////////////////////////////////
glPushMatrix();												// Save the Matrix
glBindTexture(GL_TEXTURE_2D,texture[7]);					// Which Texture
glRotatef(rs,0,1,0);										// Rotation around the Sun
glTranslatef(0,0,-39.94f);									// Translation
glRotatef(75,-1,0,0);										// Tilt of the Planet
gluSphere(sphere,1.5,30,30);								// Draw the Planet
glDisable(GL_CULL_FACE);							// Disable Face Culling for Right Around Saturn
glDisable(GL_LIGHTING);								// Disable Lighting
glDepthMask(GL_FALSE);								// Make the Depth Buffer Read-Only
	glEnable(GL_BLEND);							 	// Enable Blending
	glBlendFunc(GL_DST_COLOR,GL_ZERO);				// Blend Screen Color With Zero (Black)
	glBindTexture(GL_TEXTURE_2D, texture[18]);		// Select The Mask Texture
	glBegin(GL_QUADS);											// Start Drawing A Quad
		glTexCoord2f(0.0f,0.0f); glVertex3f(-4.0f,-4.0f,0.0f);	// Bottom Left
		glTexCoord2f(1.0f,0.0f); glVertex3f( 4.0f,-4.0f,0.0f);	// Bottom Right
		glTexCoord2f(1.0f,1.0f); glVertex3f( 4.0f, 4.0f,0.0f);	// Top Right
		glTexCoord2f(0.0f,1.0f); glVertex3f(-4.0f, 4.0f,0.0f);	// Top Left
	glEnd();
	glBlendFunc(GL_ONE, GL_ONE);					// Blend Screen With One (White)
	glBindTexture(GL_TEXTURE_2D, texture[17]);		// Select The Normal Texuture
	glBegin(GL_QUADS);											// Start Drawing A Quad
		glTexCoord2f(0.0f,0.0f); glVertex3f(-4.0f,-4.0f,0.0f);	// Bottom Left
		glTexCoord2f(1.0f,0.0f); glVertex3f( 4.0f,-4.0f,0.0f);	// Bottom Right
		glTexCoord2f(1.0f,1.0f); glVertex3f( 4.0f, 4.0f,0.0f);	// Top Right
		glTexCoord2f(0.0f,1.0f); glVertex3f(-4.0f, 4.0f,0.0f);	// Top Left
	glEnd();										// Done Drawing Ring
glEnable(GL_CULL_FACE);											// Enable Face Culling
glDepthMask(GL_TRUE);											// Depth Buffer is Writable 
glDisable(GL_BLEND);											// Disable Blending
glEnable(GL_LIGHTING);											// Enable Lighting
glPopMatrix();												// Load the Former Matrix
//////////////////////////////////////////////
////////////////////SUN///////////////////////
//////////////////////////////////////////////
glPushMatrix();												// Save the Matrix
glRotatef(r,0,0,1);												// Rotate The Sun Slowly So It Doesn't Look Static
glDisable(GL_LIGHTING);											// Disable Lighting
glDepthMask(GL_FALSE);											// Depth Buffer Is Read-Only
glEnable(GL_BLEND);												// Enable Blending
	glBlendFunc(GL_DST_COLOR,GL_ZERO);				// Blend Screen Color With Zero (Black)
	glBindTexture(GL_TEXTURE_2D, texture[1]);		// Select The Second Mask Texture
	glBegin(GL_QUADS);											// Start Drawing A Quad
		glTexCoord2f(0.0f,0.0f); glVertex3f(-10.6f,-10.6f,0.0f);	// Bottom Left
		glTexCoord2f(1.0f,0.0f); glVertex3f( 10.6f,-10.6f,0.0f);	// Bottom Right
		glTexCoord2f(1.0f,1.0f); glVertex3f( 10.6f, 10.6f,0.0f);	// Top Right
		glTexCoord2f(0.0f,1.0f); glVertex3f(-10.6f, 10.6f,0.0f);	// Top Left
	glEnd();
	glBlendFunc(GL_ONE, GL_ONE);					// Blend Screen Color With One(White)
	glBindTexture(GL_TEXTURE_2D, texture[0]);		// Select The Normal Texture
	glBegin(GL_QUADS);											// Start Drawing A Quad
		glTexCoord2f(0.0f,0.0f); glVertex3f(-10.6f,-10.6f,0.0f);	// Bottom Left
		glTexCoord2f(1.0f,0.0f); glVertex3f( 10.6f,-10.6f,0.0f);	// Bottom Right
		glTexCoord2f(1.0f,1.0f); glVertex3f( 10.6f, 10.6f,0.0f);	// Top Right
		glTexCoord2f(0.0f,1.0f); glVertex3f(-10.6f, 10.6f,0.0f);	// Top Left
	glEnd();										// Done Drawing The Sun
glDepthMask(GL_TRUE);								// Depth Buffer Is Writable
glDisable(GL_BLEND);								// Disable Blending
glEnable(GL_LIGHTING);								// Enable Lighting
glPopMatrix();										// Load the Former Matrix
}

